/*Ŀ
 ݳ                                                                      
 ݳ Program Name: MYALERT.PRG                                            
 ݳ     Language: Clipper 5.0                                            
 ݳ       Author: John Graceland                                         
 ݳ Demo Compile: clipper myalert /n/m/w/dJG_TEST                        
 ݳ      Compile: clipper myalert /n/m/w                                 
 Ĵ
 ݳ Comments:                                                            
 ݳ Substitute for Clipper's ALERT()                                     
 ݳ Make sure to include code in NewChar.obj if using TMR box type       
 Ĵ
 ݳ History:                                                             
 ݳ KSG 01/94  added SET(_SET_DEVICE,"SCREEN" )                          
 ݳ KSG 01/94  added manifest constant to remove demo function           
 ݳ KSG 01/94  added ability to use a shadow with John's alert by        
 ݳ            defining KG_SHADOW. This shadow uses Clipper's GT api     
 ݳ            and is compatible with Blinker v3.nn - protected mode     
 ݳ KSG 11/95  excluded shadow in C from current demo since the thin     
 ݳ            boxs just added look better w/o shadows                   
 
            */

#include "inkey.ch"
#include "box.ch"

// used by SetMode - you could use a SetGet function instead!
#define N_DEF_SCRN_MODE 25

#define	N_SINGLE    (CHR(218) + CHR(196) + CHR(191) + CHR(127) +     ;
                     CHR(224) + CHR(203) + CHR(192) + CHR(179) + " " )

#ifdef JG_TEST
FUNCTION Main
@maxrow(),0
dispbox(0,0,maxrow()-1,maxcol(),"","w+/gr")

? Alert( "Alert box with no choices")
? Alert( "Alert box with choices", {"OK","Not OK","Maybe","Never"} )
? Alert( "Alert box starting on row 16", {"OK","Not OK","Maybe","Never"},16 )
? Alert( "Multiple Line message:;Message one;Message two;Message three", {"OK","Not OK","Maybe","Never"})
? Alert( "Alert box with my Color", {"OK","Not OK","Maybe","Never"},,"W+/+BG,N/W")
? Alert( "Alert box with bell tone", {"OK","Not OK","Maybe","Never"},,"W+/+BG,N/W",.T. )
? Alert( "Alert box with a shadow",,,,,.T.)
RETURN (NIL)
#endif


/*
*  Function: Alert( <cMessage> [,<aOptions_>, <nTop>, <cColor>, <lBell>] )
*          :
*   Purpose: Emulates Clippers alert(), additional options listed below 
*          :
* Arguments: <cMessage>     - same as Clippers alert
*          : <aOptions_>    - array for buttons, defaults to {" Ok "}
*          : <nTop>         - Starting row position of box
*          : <cColor>       - set color
*          : <lBell>        - sound bell
*          :
*  Comments: Envoke Clippers dbugger from within Alert()
*          : Returns a numeric value equal to the option selected, 
*          : <Esc> = 0
*          :
*    Tweaks: KSG 01/94 added 
*          : SET(_SET_DEVICE,"SCREEN" )
*          : to insure output always goes to the display screen
*          :
*/

#ifndef KG_SHADOW
FUNCTION Alert( cMessage, aOptions_, nTop, cColor, lBell )
#else
FUNCTION Alert( cMessage, aOptions_, nTop, cColor, lBell, lShadow )
#endif
    local nChoice    := 0
    local nVar       := 0
    local nCol       := 0
    local nRow       := 0
    local nPromLeft  := 0
    local nBoxLeft   := 0
    local nBoxRight  := 0
    local nMsgLen    := 0
    local nBoxLen    := 0
    local nPromLen   := 0
    local cScreen    := ""
    local cOldColor  := SETCOLOR()
    local cOldWrap   := SET( _SET_WRAP , "ON" )
    local lOldCenter := SET( _SET_MCENTER, .F. )
    local lOldBlink  := SETBLINK(.F.)
    local aMessage   := {}
    local bPgUp      := SETKEY(K_PGUP, { || NIL } )
    local bPgDn      := SETKEY(K_PGDN, { || NIL } )
    local cDevice    := SET(_SET_DEVICE, "SCREEN" )

    #ifdef KG_SHADOW
    lShadow := if(valtype(lShadow) == "L", lShadow,.F.)
    #endif

    if aOptions_ == NIL
        aOptions_ := {}
        AADD( aOptions_," OK ")
    endif

    // set length of prompt line
    AEVAL( aOptions_, { |aArray| nPromLen += LEN(aArray)+2 } )

    // load message array
    while ";" $ cMessage
        // test for multiple line message
        AADD(aMessage, SUBSTR(cMessage,1,AT(";",cMessage)-1 ) )
        cMessage := STUFF(cMessage,1,AT(";",cMessage), "" )
    enddo

    if "" != cMessage
        // add remaining message string to array
        AADD(aMessage,cMessage)
    endif

    nMsgLen := LEN(aMessage)

    // determine box length based on largest message line
    AEVAL( aMessage, {|aArray| nBoxLen := MAX(nBoxLen,LEN(aArray)+4) })

    nPromLen += ( (LEN(aOptions_)+1) * 2 )
    cColor   := IF(ISCOLOR(),IF(cColor == NIL,"W+/R,W+/B",cColor),"N/W,W+/N")
    lBell    := IF( lBell == NIL, .F., lBell )
    nTop     := IF( nTop == NIL, 10, nTop )

    // test for the larger of box lenght vs. length of prompt line
    nBoxLen   := IF( nBoxLen > nPromLen, nBoxLen, nPromLen + 4 )

    nBoxLeft  := ( MAXCOL() - nBoxLen ) / 2
    nBoxRight := nBoxLeft + nBoxLen
    nPromLeft := ( ( MAXCOL()-nPromLen ) / 2)

    // save the original screen
    #ifndef KG_SHADOW
    cScreen := SAVESCREEN(nTop,nBoxLeft,nTop +3 +nMsgLen, nBoxRight +1 )
    #else
    cScreen := SAVESCREEN(nTop, nBoxLeft, nTop +4 +nMsgLen, nBoxRight +2 )
    #endif

    DISPBEGIN()

    SETCOLOR( cColor )

    // display Alert box with no TMR if from the errorsystem
    // or use thin style box if not from DefError()
    if procname(1) == "DEFERROR"
        setmode(N_DEF_SCRN_MODE)
        @nTop,nBoxLeft,nTop +3 +nMsgLen,nBoxRight +1 BOX B_SINGLE + " "
    else
        @nTop,nBoxLeft,nTop +3 +nMsgLen,nBoxRight +1 BOX N_SINGLE
        @nTop,nBoxLeft   say padr("",nBoxRight - nBoxLeft +2) color "w+*/w"
        @nTop,nBoxLeft   say chr(213)                          color "w+/n"
        @nTop,nBoxLeft+1 say chr(214)                          color "w+/n"
        @nTop,nBoxLeft+2 say chr(179)                          color "n/w+*"
    endif

    #ifdef KG_SHADOW
        if lShadow
            shadow(nTop,nBoxLeft, nTop +3 +nMsgLen, nBoxRight +1,2)
        endif
    #endif
    for nVar := 1 to nMsgLen
        @nTop +nVar,nBoxLeft +1 say padc(aMessage[nVar],nBoxLen,SPACE(1))
    next

    nCol := nPromLeft +3

    // Display the prompt line
    nRow := ROW() +2

    for nVar := 1 to len( aOptions_ )
        @ nRow, nCol PROMPT SPACE(1) + aOptions_[nVar] + SPACE(1)
        nCol += LEN(aOptions_[nVar]) +4
    next

    DISPEND()

    if lBell
        TONE(499,2)
        TONE(299,1)
    endif

    MENU TO nChoice

    // reset environment
    SET( _SET_WRAP   , cOldWrap )
    SET( _SET_MCENTER, lOldCenter )
    SETBLINK( lOldBlink )
    SET(_SET_DEVICE, cDevice )

    #ifndef KG_SHADOW
    RESTSCREEN( nTop, nBoxLeft, nTop +3 +nMsgLen, nBoxRight +1,cScreen )
    #else
    RESTSCREEN( nTop, nBoxLeft, nTop +4 +nMsgLen, nBoxRight +2,cScreen )
    #endif

    SETCOLOR( cOldColor )

    SETKEY(K_PGUP, bPgUp )
    SETKEY(K_PGDN, bPgDn )

RETURN IF( ( LASTKEY() == K_ESC  .OR. ;
             LASTKEY() == K_PGUP .OR. ;
             LASTKEY() == K_PGDN    ),;
             0,nChoice;
)











