/*
 * -------------------------------------------------------------------------*
 * KSG 6/93
 * Added ft_shadow to the menu function
 *
 * -------------------------------------------------------------------------*
 * File......: VERTMENU.PRG
 * Author....: Greg Lief
 * CIS ID....: 72460,1760
 * Date......: $Date:   15 Aug 1991 23:04:48  $
 * Revision..: $Revision:   1.1  $
 * Log file..: $Logfile:   E:/nanfor/src/vertmenu.prv  $
 *
 * This function is an original work by Mr. Grump and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log:   E:/nanfor/src/vertmenu.prv  $
 * 
 *    Rev 1.1   15 Aug 1991 23:04:48   GLENN
 * Forest Belt proofread/edited/cleaned up doc
 * 
 *    Rev 1.0   01 Apr 1991 01:02:26   GLENN
 * Nanforum Toolkit
 *
 */

/*  $DOC$
 *  $FUNCNAME$
 *     FT_MENU2()
 *  $CATEGORY$
 *     Menus/Prompts
 *  $ONELINER$
 *     Vertical lightbar menu
 *  $SYNTAX$
 *     FT_MENU2( <aMenuarray> [, <cColors> ] ) -> NIL
 *  $ARGUMENTS$
 *     <aMenuarray> is an array of menu options, messages, and action
 *      blocks.
 *
 *     Each element in this array is a nested array with the structure:
 *
 *        element[x, 1] = menu option
 *        element[x, 2] = message to be displayed when option is highlighted
 *        element[x, 3] = code block to be executed when option is selected
 *
 *     <cColors> is a string containing colors for the prompts, in the same
 *     format as that returned by Set( _SET_COLOR ).  If not supplied,
 *     colors default to the current color setting.
 *  $RETURNS$
 *     NIL
 *  $DESCRIPTION$
 *     This function greatly simplifies the process of displaying light-bar
 *     menus.  All prompts are padded out with spaces so they are the same
 *     length, a box is drawn around the prompts, the box is automatically
 *     centered on the screen, and the underlying screen is restored after
 *     a menu selection has been made.
 *
 *     Additionally, because you can tie action blocks to each menu
 *     option, you can save on a lot of DO CASE or IF..ELSEIF code in your
 *     main program.  See the test code for a succinct demonstration.
 *  $EXAMPLES$
 *      LOCAL mainmenu := ;
 *          { { "Data Entry", "Enter data",   { || FT_MENU2(datamenu)  } }, ;
 *            { "Reports",    "Hard copy",    { || FT_MENU2(repmenu)   } }, ;
 *            { "Maintenance","Reindex files",{ || FT_MENU2(maintmenu) } }, ;
 *            { "Quit", "See ya later" } }
 *      FT_MENU2(mainmenu)
 *  $END$
 */

#include "box.ch"

#define	N_SINGLE    (CHR(218) + CHR(196) + CHR(191) + CHR(127) +    ;
                     CHR(224) + CHR(203) + CHR(192) + CHR(179) + " ")


// test code

#ifdef FT_TEST
#xcommand FT_BYEBYEBOX()    => FT_XBOX(.T.)
FUNCTION MAIN
LOCAL nCursor  := setcursor(0)
LOCAL MAINMENU := ;
    { { "DATA ENTRY", "ENTER DATA",         { || FT_MENU2(datamenu,,.t.)}}, ;
      { "Reports",    "Hard copy",          { || FT_MENU2(repmenu,,.t.,2)}},;
      { "Maintenance","Reindex files, etc.",{ || FT_MENU2(maintmenu,,.t.)}},;
      { "Quit", "See ya later" } }

local datamenu := { { "Customers","Item one"   , { || cust() } }   , ;
                    { "Invoices", "Item two"   , { || inv() } }    , ;
                    { "Vendors",  "Item three" , { || vendors() } }, ;
                    { "Exit", "Return to Main Menu" } }

local repmenu :=  { { "Customer List", , { || custrep() } }  , ;
                    { "Past Due",      , { || pastdue() } }  , ;
                    { "Weekly Sales",  , { || weeksales() } }, ;
                    { "Monthly P&L",   , { || monthpl() } }  , ;
                    { "Vendor List",   , { || vendorrep() } }, ;
                    { "Exit", "Return to Main Menu" } }

local maintmenu := { { "Reindex",  "Rebuild index files", { || re_ntx() } } , ;
                     { "Backup",   "Backup data files"  , { || backup() } } , ;
                     { "Compress", "Compress data files", { || compress()} }, ;
                     { "Exit", "Return to Main Menu" } }
dispbox(0,0,maxrow(),maxcol(),"","w+/b")

kg_init()

FT_XBOX('L','W','D','GR+/R','W+/R',1,10,'This is a test of two modified',;
                  'functions from NANFOR.LIB',;
                  'Press any key to test them out')
// restore old screen
FT_BYEBYEBOX()

FT_MENU2(mainmenu,"w+/rb,b/w",.t.,4)

FT_XBOX(,'W',,'GR+/BG','W+/BG',,,;
            'Make sure to include the following for enchanced xbox',;
            '#xcommand FT_BYEBYEBOX()    => FT_XBOX(.T.)',;
            'for removing box from the screen',;
            'Press any key to test them out')
// restore old screen
FT_BYEBYEBOX()
@maxrow(),0
setpos(maxrow()-1,0)
setcursor(nCursor)
setmode(25)
return nil

/* stub functions to avoid missing symbols */
/* KSG 06/93 -- added return to each fake udf */
static function cust
return nil
static function inv
return nil
static function vendors
return nil
static function custrep
return nil
static function pastdue
return nil
static function weeksales
return nil
static function monthpl
return nil
static function vendorrep
return nil
static function re_ntx
return nil
static function backup
return nil
static function compress
return nil

#endif

/*
   FT_MENU2(): display vertical menu
*/

FUNCTION ft_menu2( aMenuInfo, cColors, lShadow, nShadow )

LOCAL nChoice     := 1                       ,;
      nOptions    := Len( aMenuInfo )        ,;
      nMaxwidth   := 0                       ,;
      nLeft                                  ,;
      x                                      ,;
      cOldscreen                             ,;
      nTop                                   ,;
      lOldwrap    := Set( _SET_WRAP, .T. )   ,;
      lOldcenter  := Set( _SET_MCENTER, .T. ),;
      lOldmessrow := Set( _SET_MESSAGE )     ,;
      cOldcolor   := Set( _SET_COLOR )

IF cColors # NIL
   Set( _SET_COLOR, cColors )
ENDIF

/* 
* KSG 06/93 added option to use a shadow
* The default is no shadow, so that it conforms
* to older code. Note the color option to ft_shadow
* is not used, and uses the default shadow color
*/

lShadow := if(valtype( lShadow ) == "L", lShadow,.F.)

/* if no message row has been established, use bottom row */
IF lOldmessrow == 0
   Set( _SET_MESSAGE, Maxrow() )
ENDIF

/* determine longest menu option */
Aeval( aMenuInfo, { | ele | nMaxwidth := max( nMaxwidth, len( ele[1] ) ) } )

/* establish top and left box coordinates */
nLeft := ( ( Maxcol() + 1 ) - nMaxwidth ) / 2
nTop  := ( ( Maxrow() + 1 ) - ( nOptions + 2 ) ) / 2

DO WHILE nChoice != 0 .AND. nChoice != nOptions

   cOldscreen := Savescreen( nTop, nLeft - 1, nTop + nOptions + 2, nLeft + nMaxwidth+2 )


// @ nTop, nLeft - 1, nTop + nOptions + 1, nLeft + nMaxwidth BOX B_SINGLE + ' '
   @ nTop, nLeft - 1, nTop + nOptions + 1, nLeft + nMaxwidth BOX N_SINGLE + ' '
   
   
   if lShadow
      // draw shadow if requested
      if valtype(nShadow) == "N"
          nShadow := if(nShadow >0 .and. nShadow <= 8,nShadow,8)
      else
          nShadow := 8
      endif

      ft_shadow(nTop, nLeft -1,nTop +nOptions +1 ,nLeft +nMaxwidth,nShadow )
   endif
   
   Devpos( nTop, nLeft )
   FOR x := 1 to Len( aMenuInfo )
      IF Len( aMenuInfo[x] ) > 1 .AND. aMenuInfo[x,2] != NIL
         @ Row() + 1, nLeft PROMPT Padr( aMenuInfo[x, 1], nMaxwidth ) ;
                            MESSAGE padc(aMenuInfo[x,2],maxcol() +1)
      ELSE
         @ Row() + 1, nLeft PROMPT Padr( aMenuInfo[x,1], nMaxwidth )
      ENDIF
   NEXT

   MENU TO nChoice

   Restscreen( nTop, nLeft - 1, nTop + nOptions + 2, nLeft + nMaxwidth+2 , cOldscreen )

   /* execute action block attached to this option if there is one */
   IF nChoice > 0 .AND. Len(  aMenuInfo[ nChoice ]  ) == 3
      Eval(  aMenuInfo[nChoice,3]  )
   ENDIF

ENDDO

/* restore previous message and wrap settings */
Set( _SET_MESSAGE, lOldmessrow )
Set( _SET_MCENTER, lOldcenter )
Set( _SET_WRAP,    lOldwrap )
Set( _SET_COLOR,   cOldcolor )

RETURN NIL

* end of file: vertmenu.prg

/*
*   Routine: kg_init
*          : 
*   Purpose: wrapper for David Tessitore's C-code for redefining any ascii
*          : characters which i use for TMR look ala Norton Utilities.
*          : 
* Arguments: void
*          : 
*  Comments: locals were used to keep code on the screen
*          : 
*/
procedure kg_init
    local x1 :=   0
    local x2 :=   1
    local x3 :=   8
    local x4 :=  16
    local x5 := 128
    local x6 := 143
    local x7 := 241
    local x8 := 255

    //         +-- character to redefine
    //         |     +-- array to make new pixel pattern for specified char
    //         |     |
    KG_Chars( 127, {x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2} )
    KG_Chars( 224, {x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x8} )
    KG_Chars( 179, {x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5} )
    KG_Chars( 181, {x2,x2,x2,x2,x2,x2,x2,x8,x2,x2,x2,x2,x2,x2,x2,x2} )
    KG_Chars( 191, {x8,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2,x2} )
    KG_Chars( 192, {x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x8} )
    KG_Chars( 195, {x5,x5,x5,x5,x5,x5,x5,x8,x5,x5,x5,x5,x5,x5,x5,x5} )
    KG_Chars( 196, {x8,x1,x1,x1,x1,x1,x1,x1,x1,x1,x1,x1,x1,x1,x1,x1} )
    KG_Chars( 200, {x1,x1,x1,x1,x1,x1,x1,x8,x1,x1,x1,x1,x1,x1,x1,x1} )
    KG_Chars( 203, {x1,x1,x1,x1,x1,x1,x1,x1,x1,x1,x1,x1,x1,x1,x1,x8} )
    KG_Chars( 209, {x8,x4,x4,x4,x4,x4,x4,x4,x4,x4,x4,x4,x4,x4,x4,x4} )
    KG_Chars( 210, {x4,x4,x4,x4,x4,x4,x4,x4,x4,x4,x4,x4,x4,x4,x4,x4} )
    KG_Chars( 211, {x4,x4,x4,x4,x4,x4,x4,x8,x1,x1,x1,x1,x1,x1,x1,x1} )
    KG_Chars( 212, {x4,x4,x4,x4,x4,x4,x4,x8,x4,x4,x4,x4,x4,x4,x4,x4} )
    KG_Chars( 213, {x8,x5,x5,x5,x5,x5,x6,x6,x5,x5,x5,x5,x5,x5,x5,x8} )
    KG_Chars( 214, {x8,x2,x2,x2,x2,x2,x7,x7,x2,x2,x2,x2,x2,x2,x2,x8} )
    KG_Chars( 218, {x8,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5,x5} )
    // below are for the lowered top tin box
    KG_Chars( 160, {x1,x1,x1,x1,x1,x1,x8,x5,x5,x5,x5,x5,x5,x5,x5,x5} )
    KG_Chars( 161, {x1,x1,x1,x1,x1,x1,x8,x1,x1,x1,x1,x1,x1,x1,x1,x1} )
    KG_Chars( 162, {x1,x1,x1,x1,x1,x1,x8,x2,x2,x2,x2,x2,x2,x2,x2,x2} )
    
return

